#pragma once

#include "global_hotkey_global.h"
#include <QObject>
#include <QMultiHash>
#include <QAbstractNativeEventFilter>
#include <QKeySequence>

class GLOBAL_HOTKEY_EXPORT QGlobalShortcut : public QObject {
    Q_OBJECT
    Q_PROPERTY(QKeySequence key READ key WRITE setKey)

private:

    class QGlobalShortcutEventFilter : public QAbstractNativeEventFilter {
    public:
        bool nativeEventFilter(const QByteArray& eventType, void* message, long* result);
    };

public:
    explicit QGlobalShortcut(QObject* parent = nullptr);
    explicit QGlobalShortcut(const QKeySequence& keyseq, QObject* parent = nullptr);
    ~QGlobalShortcut();
    bool nativeEventFilter(const QByteArray& event_type, void* message, long* result);

    QKeySequence key() const;
    // you must unset hot key explicitly
    bool setKey(const QKeySequence& keyseq);
    bool unsetKey();

signals:
    void activated();

    void error(QString);

private:
    QKeySequence keyseq_;
    void initialize();

private:
    static QGlobalShortcutEventFilter global_shortcut_event_;
    static QMultiHash<quint32, QGlobalShortcut*> shortcuts_;
    static bool activate(quint32 id);
    static inline quint32 calcId(const QKeySequence& keyseq);
#ifdef Q_OS_WIN
    static quint32 calcId(quint32 k, quint32 m) {
        return k | m;
    }
#else
    static quint32 calcId(quint32 k, quint32 m) {
        return k | (m << 16);
    }
#endif
    static inline Qt::Key getKey(const QKeySequence& keyseq);
    static inline Qt::KeyboardModifiers getMods(const QKeySequence& keyseq);
    static quint32 toNativeKeycode(Qt::Key k);
    static quint32 toNativeModifiers(Qt::KeyboardModifiers m);
#ifdef Q_OS_WIN
    static QString getErrorMsg();
#endif
    static bool registerKey(quint32 k, quint32 m, quint32 id);
    static bool unregisterKey(quint32 k, quint32 m, quint32 id);
};
