#include "popup.h"
#include <QLabel>
#include <QGridLayout>
#include <QPropertyAnimation>
#include <QTimer>
#include <QApplication>
#include <QDesktopWidget>
#include <QPainter>
#include <QRect>

PopUp::PopUp(QWidget *parent)
    : QWidget{parent}
{
    mLabel = new QLabel{};
    mLayout = new QGridLayout{this};
    mAnimation = new QPropertyAnimation{this};
    mTimer = new QTimer{this};

    setWindowFlags(Qt::FramelessWindowHint | Qt::Tool | Qt::WindowStaysOnTopHint);
    setAttribute(Qt::WA_TranslucentBackground);
    setAttribute(Qt::WA_ShowWithoutActivating);

    mAnimation->setTargetObject(this);
    mAnimation->setPropertyName("popUpOpacity");
    connect(mAnimation, &QAbstractAnimation::finished, this, &PopUp::hide);

    mLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    mLabel->setStyleSheet("QLabel { color : white; "
                          "margin-top: 10px;"
                          "margin-bottom: 10px;"
                          "margin-left: 10px;"
                          "margin-right: 10px; }");

    mLayout->addWidget(mLabel, 0, 0);
    setLayout(mLayout);
    connect(mTimer, &QTimer::timeout, this, &PopUp::hideAnimation);
}

void PopUp::setOpacity(float opacity)
{
    mOpacity = opacity;
    setWindowOpacity(opacity);
}

float PopUp::opacity() const
{
    return mOpacity;
}

void PopUp::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)

    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QRect roundedRect;
    roundedRect.setX(rect().x() + 5);
    roundedRect.setY(rect().y() + 5);
    roundedRect.setWidth(rect().width() - 10);
    roundedRect.setHeight(rect().height() - 10);

    painter.setBrush(QBrush(QColor(0,0,0,180)));
    painter.setPen(Qt::NoPen);

    painter.drawRoundedRect(roundedRect, 10, 10);
}

void PopUp::setPopupText(const QString &text)
{
    mLabel->setText(text);
    this->adjustSize();
}

void PopUp::show()
{
    setWindowOpacity(0.0);

    mAnimation->setDuration(150);
    mAnimation->setStartValue(0.0);
    mAnimation->setEndValue(1.0);

    auto geo = QApplication::desktop()->availableGeometry();
    setGeometry(geo.width() - 30 - width() + geo.x(),
                geo.height() - 30 - height() + geo.y(),
                width(),
                height());
    QWidget::show();

    mAnimation->start();
    mTimer->start(3000);
}

void PopUp::hideAnimation()
{
    mTimer->stop();
    mAnimation->setDuration(1000);
    mAnimation->setStartValue(1.0);
    mAnimation->setEndValue(0.0);
    mAnimation->start();
}

void PopUp::hide()
{
    if(mOpacity < 0.01) {
        QWidget::hide();
    }
}
